--[[---------------------------------------------------------------------------

Copyright (c) 2008-2017 by K. Scott Piel 
All Rights Reserved

E-mail: < kscottpiel@gmail.com >

This file is part of nUI.

The author no longer develops or directly supports this addon and has released it
into the public domain under the Creative Commons Attribution-NonCommercial 3.0 
Unported license - a copy of which is included in this distribution. This source
may not be distributed without said license.
	   
nUI is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; 
without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR 
PURPOSE.  See the enclosed license for more details.
	
THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS  "AS IS" AND 
ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT  LIMITED TO, THE IMPLIED 
WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. 
IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, 
INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, 
BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, 
DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF 
LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE 
OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED 
OF THE POSSIBILITY OF SUCH DAMAGE.

--]]---------------------------------------------------------------------------
if not nUI_Options then nUI_Options = {}; end

-- Default Options, if at any time these get nullified, they will reset to these defaults ( XP on left, Rep on right)
nUI_Options.Trackers = {}
nUI_Options.Trackers[1] = 1
nUI_Options.Trackers[2] = 2

--Define tracker frame 
local nUI_Trackers = CreateFrame("Frame",nil,WorldFrame)

-- Skins (Make sure you re-assign this if you change the layout)
nUI_Trackers.Skins = {}
nUI_Trackers.Skins[1] = nUI_DefaultConfig.TrackerBars["Bar1"]
nUI_Trackers.Skins[2] = nUI_DefaultConfig.TrackerBars["Bar2"]

-- Buttons to change the bar settings
nUI_Trackers.Buttons = {
    [1] = CreateFrame( "Button", "nUI_Tracker_Controller1", nUI_Dashboard.Anchor, "UIPanelButtonTemplate" ),
    [2] = CreateFrame( "Button", "nUI_Tracker_Controller2", nUI_Dashboard.Anchor, "UIPanelButtonTemplate" ),
}

-- Trackers available
nUI_Trackers.Bars = {    
    [1] = "nUI_XPBar",
    [2] = "nUI_FactionBar",
   -- [3] = "nUI_HonorBar",
}

nUI_Trackers.Captions = {
    [1] = POWER_TYPE_EXPERIENCE,
    [2] = REPUTATION ,
   -- [3] = HONOR,
}

-- Override coloring, these are changed before applying skin again
nUI_Trackers.Colors = {
    [1] = 
    {
        rested = { r = 0.25, g = 1, b = 0.25 },
        xp     = { r = 0.25, g = 0.5, b = 1 },
    },
    [2] = { r = 0.25, g = 0.50, b = 1.0 },
   --[3] = { r = 1.00, g = 0.24, b = 0.0 },
}

-------------------------------------------------------------------------------
-- This should allow addons to add new trackers
-- Bear in mind that not only will your bars have to follow nUI statusBar settings ( apply and set functions etc)
-- But it will also need a bar.setActive function that will call its update display routine
-- This has clearly not been tested yet so may be warrant to change or removal dependant on useability and ease of use
function nUI_Trackers.AddNewTracker(button, trackerName, menuText, rgbColor)
    while nUI_Trackers.busy do end
    nUI_Trackers.busy = true
    local nextBar = #nUI_Trackers.Bars
    local nextButton = #nUI_Trackers.Buttons
    nUI_Trackers.Buttons[nextButton] = button
    nUI_Trackers.Bars[nextBar] = trackerName
    nUI_Trackers.Captions[nextBar] = menuText
    nUI_Trackers.Colors[nextBar] = rgbColor
    nUI_Trackers.busy = false
end

-------------------------------------------------------------------------------
-- Update the button with its associated trackerID
function nUI_Trackers.UpdateButton(buttonID, trackerID)
    local button = nUI_Trackers.Buttons[buttonID] or nil
    if not button then return end
    button.type = trackerID or 0
    button.Text:SetText(trackerID or 0)
    nUI_Options.Trackers[buttonID] = trackerID or 0    
end

-------------------------------------------------------------------------------
-- Deactivate the selected tracker and unassociate it from its last bar
function nUI_Trackers.Deactivate(tracker)
    
    if tracker then
        tracker.barID = nil
        tracker.setEnabled(false)
        tracker:Hide()
    end

end

-------------------------------------------------------------------------------
-- Activate the selected tracker and associate it with the selected bar
function nUI_Trackers.Activate(tracker, barID)
    if not tracker then return end
    local skin = nUI_Trackers.Skins[barID]
    local anchor = skin.barAnchor 
    tracker.anchor = anchor
    tracker.barID = barID or 0
    tracker.setEnabled(true)     
    if tracker.options then
        tracker.options.width = skin.options.barWidth
    end
    tracker.applyScale()
    tracker.applyAnchor()     
    tracker.setActive()
    tracker:Show()
end

-------------------------------------------------------------------------------
-- Set up new Associations and Remove old ones
function nUI_Trackers.Reassociate(trackerID, buttonID)
    
    -- Remove the old association
    local tracker = _G[nUI_Trackers.Bars[trackerID]]
    if tracker and tracker.barID and tracker.barID > 0 then
        nUI_Trackers.UpdateButton(tracker.barID, 0)
    end
    
    -- Add the new association
    nUI_Trackers.UpdateButton(buttonID, trackerID)
end

-------------------------------------------------------------------------------
-- Apply Options - call whenever the options are being read or changed
function nUI_Trackers.ApplyOptions()
    
    if InCombatLockdown() then return end
    
    -- Go through each tracker bar setting in the options and update the bars and buttons accordingly
    for barID, trackerID in pairs(nUI_Options.Trackers) do

        -- Update the button associated with this bar
        nUI_Trackers.UpdateButton(trackerID, barID)
        
        -- If we have a tracker then activate it
        local tracker = _G[nUI_Trackers.Bars[trackerID]] or nil
        if tracker then             
            nUI_Trackers.Activate(tracker, barID)
        end
    end
end

-----------------------------------------------------------------------------
-- Initialise Trackers with our customised settings
function nUI_Trackers.InitialiseTrackers()
    for trackerID, trackerName in pairs(nUI_Trackers.Bars) do    
        local tracker = _G[trackerName]
        if tracker and tracker.options then
            local skin = tracker.barID and nUI_Trackers.Skins[barID]
            if skin then                
                tracker.options.width = skin.options.barWidth
            end
            if tracker.options.bar then
                tracker.options.bar.colors = nUI_Trackers.Colors[trackerID]
            end
        end 
    end
end

-------------------------------------------------------------------------------
-- Define drop down menu for bar selection
nUI_Trackers.DropDownMenu = CreateFrame("Frame", "nUI_Trackers.DropDownMenu")
nUI_Trackers.DropDownMenu.displayMode = "MENU"
local info = {}

nUI_Trackers.DropDownMenu.initialize = function(self, level) 
    if not level then return end
    local buttonID = nUI_Trackers.DropDownMenu.button
    if not buttonID then return end
    
    local button = nUI_Trackers.Buttons[buttonID]
    local config = nUI_Trackers.Skins[buttonID]
    local curTracker = _G[nUI_Trackers.Bars[button.type]] or nil
    
    wipe(info)
    if level == 1 then
        -- Create the title of the menu
        info.isTitle = 1
        info.text = MINIMAP_TRACKING_TOOLTIP_NONE
        info.notCheckable = 1
        UIDropDownMenu_AddButton(info, level)
        
        info.disabled     = nil
        info.isTitle      = nil
        info.notCheckable = nil
        
        for trackerID, trackerName in pairs(nUI_Trackers.Captions) do
            local thisTracker = _G[nUI_Trackers.Bars[trackerID]]
            
            -- Make sure the tracker for this item is a valid frame
            if thisTracker then
                
                -- set values for this menu item
                info.text = trackerName
                info.disabled = not thisTracker.isTrackable() --thisTracker.barID and thisTracker.barID ~= 0
                info.checked = nUI_Options.Trackers[buttonID] == trackerID
                info.notCheckable = false
                
                -- define function to execute when clicked ( doesn't work outside this code block for some reason )
                info.func = function()                                

                    -- If  this items tracker that we clicked already was associated with a bar then remove associations
                    nUI_Trackers.Reassociate(trackerID, buttonID)

                    -- if there was already a tracker associated then turn it off so it can be associated elsewhere
                    nUI_Trackers.Deactivate(curTracker)
                    nUI_Trackers.Activate(thisTracker,buttonID)

                end
                UIDropDownMenu_AddButton(info, level)            
            end
        end

        info.text = RESET_TO_DEFAULT 
        info.disabled = false
        info.checked = false
        info.notCheckable = true
        info.func = function()
            nUI_Options.Trackers = {}
            nUI_Options.Trackers[1] = 1
            nUI_Options.Trackers[2] = 2
            nUI_Trackers.ApplyOptions()            
        end
        UIDropDownMenu_AddButton(info, level)                
        
        info.text = OBJECTIVES_STOP_TRACKING 
        info.disabled = button.type == 0 or button.type == nil
        info.checked = false
        info.notCheckable = true
        info.func = function()
            nUI_Trackers.Deactivate(curTracker)
            nUI_Trackers.UpdateButton(buttonID)
        end    
        UIDropDownMenu_AddButton(info, level)            

        -- Close menu item
        info.text         = CLOSE
        info.func         = function() CloseDropDownMenus() end
        info.disabled     = false
        info.checked      = nil
        info.notCheckable = 1
        UIDropDownMenu_AddButton(info, level)
        
        
        
    end
end


-------------------------------------------------------------------------------
-- Initialise the Controller Buttons
function nUI_Trackers.InitialiseButtons(buttonID)
    
    -- Check each Button and ensure the bar and tracker is attributed to it properly
    -- And then position the button and set the OnEnter, OnLeave and OnClick scripts
    for barID, barButton in pairs(nUI_Trackers.Buttons) do

        barButton.bar = barID
        barButton.type = nUI_Options.Trackers[barID] or 0
        local config = nUI_Trackers.Skins[barID]
        
        
        if not buttonID or buttonID == barID then             
            
            -- Set the text info of the button
            barButton.Text:SetFontObject("GameFontNormalSmall")
            barButton.Text:SetText(barButton.type)
            
            -- Set the size of the button
            barButton:SetWidth(config.options.buttonWidth)
            barButton:SetHeight(config.options.buttonHeight)
            
            -- Set and apply the anchor for the button
            local anchor = config.buttonAnchor
            barButton:ClearAllPoints()
            barButton:SetPoint(anchor.anchor_pt, anchor.relative_to, anchor.relative_pt, anchor.xOfs, anchor.yOfs)
            
            -- Set up tooltip display 
            barButton:SetScript( "OnEnter",
                function()
                    if not InCombatLockdown() then
                        GameTooltip:SetOwner( barButton )
                        GameTooltip:SetText( nUI_Trackers.Captions[barButton.type] or NONE, 1, 0.83, 0 )
                        GameTooltip:Show()
                    end
                end
            )
            
			barButton:SetScript( "OnLeave", 
				function() 
                    if not InCombatLockdown() then
					   GameTooltip:Hide()
                    end
				end 
			);

            -- Register click button and set the callback for clicking
            barButton:RegisterForClicks( "AnyUp" )
            barButton:SetScript( "OnClick", 
                function( self, button, down )                    
                    nUI_Trackers.DropDownMenu.button = barID
                    ToggleDropDownMenu(1, nil, nUI_Trackers.DropDownMenu, barButton:GetName(), 0, 0)
                end
            )
            
            barButton:Show()
        end
    end
end

-------------------------------------------------------------------------------
-- Initialise Bars
function nUI_Trackers.InitialiseBars()
    nUI_Trackers.ApplyOptions()
end

-------------------------------------------------------------------------------
-- Validate Tracker Options to make sure that we have options to choose from
function nUI_Trackers.ValidateOptions()

    -- If no trackers are set at all ( not 0 but nil) then reset to defaults
    if not nUI_Options.Trackers then
        nUI_Options.Trackers = {}
        nUI_Options.Trackers[1] = 1
        nUI_Options.Trackers[2] = 2
    end    

end
    
-------------------------------------------------------------------------------
-- Enable buttons dependent on combat situation to ensure no combat related errors (crosses fingers)
function nUI_Trackers.EnableButtons(enable)
    for barID,barButton in pairs(nUI_Trackers.Buttons) do
        if enable then
            barButton:Enable()
        else
            barButton:Disable()
        end
    end
end

-------------------------------------------------------------------------------
-- Handle events
local function onTrackerBarEvent( who, event, arg1,arg2)
		
    if event == "VARIABLES_LOADED" then
        nUI_Trackers.ValidateOptions()
        
	elseif event == "PLAYER_ENTERING_WORLD" then 
        nUI_Trackers.ValidateOptions()
        nUI_Trackers.InitialiseTrackers()
        nUI_Trackers.InitialiseButtons()
        nUI_Trackers.InitialiseBars()
        
    elseif event == "PLAYER_REGEN_ENABLED" then        
        nUI_Trackers.EnableButtons(true)
    
    elseif event == "PLAYER_REGEN_DISABLED" then
        nUI_Trackers.EnableButtons(false)

    end
    
end

nUI_Trackers:SetScript( "OnEvent", onTrackerBarEvent );
nUI_Trackers:RegisterEvent( "VARIABLES_LOADED");
nUI_Trackers:RegisterEvent( "PLAYER_ENTERING_WORLD" );
nUI_Trackers:RegisterEvent( "PLAYER_REGEN_ENABLED")
nUI_Trackers:RegisterEvent( "PLAYER_REGEN_DISABLED" );
